#include <owl/pch.h>
#include <owl/applicat.h>
#include <mmsystem.h>
#include <windows.h>
#include <winuser.h>
#include <owl/groupbox.h>
#include <owl/radiobut.h>
#include <string.h>

#define BUF_SIZE  500

WAVEINCAPS wavecaps;
char temp[10];
char caption[100];
UINT freq = 11025;

void SetupCaption()
{
  strcpy(caption, "Recorder - [ ");
  strcat(caption, wavecaps.szPname);
  ltoa(freq, temp, 10);
  strcat(caption, " ");
  strcat(caption, temp);
  strcat(caption, "Hz ]");
}

class TSetDialog: public TDialog {
    TRadioButton *b1, *b2, *b3;
    TGroupBox *group;
  public:
    TSetDialog(TWindow* parent, TResId resId):
               TDialog(parent, resId)
    {
      group = new TGroupBox(this, 101);
      b1 = new TRadioButton(this, 102, group);
      b2 = new TRadioButton(this, 103, group);
      b3 = new TRadioButton(this, 104, group);
    }
    void SetupWindow()
    {
      TDialog::SetupWindow();
      switch (freq)
      {
        case 11025: b1 -> Check(); break;
        case 22050: b2 -> Check(); break;
        case 44100: b3 -> Check(); break;
      }
    }
    void CmOk()
    {
      TDialog::CmOk();

      if (b1 -> GetCheck())
        freq = 11025;
      else
      if (b2 -> GetCheck())
        freq = 22050;
      else
        freq = 44100;
    }
    ~TSetDialog()
    {
      delete b3;
      delete b2;
      delete b1;
      delete group;
    }
    DECLARE_RESPONSE_TABLE(TSetDialog);
};

DEFINE_RESPONSE_TABLE1(TSetDialog, TDialog)
  EV_CHILD_NOTIFY(IDOK, BN_CLICKED, CmOk),
END_RESPONSE_TABLE;

class TMainWindow: public TWindow {
    HWAVEIN h;
    WAVEHDR buf;
    WAVEFORMATEX wavefmt;
    void Init();
  public:
    TMainWindow();
    void SetupWindow();
         LRESULT DataReady(unsigned int wParam, long lParam);
    void Paint(TDC& dc, bool erase, TRect& rect);
    void EvRButtonDown(uint modKeys, TPoint& point);
    ~TMainWindow()
    {
      waveInClose(h);
      delete buf.lpData;
    }
  DECLARE_RESPONSE_TABLE(TMainWindow);
};

DEFINE_RESPONSE_TABLE1(TMainWindow, TWindow)
  EV_MESSAGE(MM_WIM_DATA, DataReady),
  EV_WM_PAINT,
  EV_WM_RBUTTONDOWN,
END_RESPONSE_TABLE;

TMainWindow::TMainWindow(): TWindow(0, 0, 0)
{
}

void TMainWindow::Init()
{
  waveInClose(h);

  wavefmt.wFormatTag = WAVE_FORMAT_PCM;
  wavefmt.nChannels = 1;
  wavefmt.nSamplesPerSec = freq;
  wavefmt.nAvgBytesPerSec = BUF_SIZE;
  wavefmt.nBlockAlign = 2;
  wavefmt.wBitsPerSample = 8;
  wavefmt.cbSize = 0;

  MMRESULT err = waveInOpen(&h, 0, &wavefmt, (DWORD)HWindow, 0, CALLBACK_WINDOW);
  char errstr[100] = "Error opening waveform-audio input device!\nerror ";

  if (err != MMSYSERR_NOERROR)
  {
    ltoa(err, temp, 10);
    strcat(errstr, temp);
    MessageBox(errstr, "Error", MB_OK | MB_ICONSTOP);
  }
}

void TMainWindow::SetupWindow()
{
  TWindow::SetupWindow();
  SetBkgndColor(TColor::Black);

  Init();

  buf.lpData = new char[BUF_SIZE];
  buf.dwBufferLength = BUF_SIZE;
  buf.dwUser = 0;

  waveInPrepareHeader(h, &buf, sizeof(buf));
  waveInAddBuffer(h, &buf, sizeof(buf));
  waveInStart(h);
}

LRESULT TMainWindow::DataReady(unsigned int wParam, long lParam)
{
  waveInPrepareHeader(h, &buf, sizeof(buf));
  waveInAddBuffer(h, &buf, sizeof(buf));
  InvalidateRect(GetClientRect(), TRUE);
  return 0;
}

void TMainWindow::Paint(TDC& dc, bool erase, TRect& rect)
{
  TRect r;

  GetClientRect(r);
  dc.SelectObject((HPEN)GetStockObject(WHITE_PEN));
  dc.MoveTo(0, (r.bottom + (unsigned char)buf.lpData[0] - 128) >> 1);

  for (int i = 1; i < Attr.W; i++)
    dc.LineTo(i, (r.bottom + (unsigned char)buf.lpData[i] - 128) >> 1);
}

void TMainWindow::EvRButtonDown(uint modKeys, TPoint& point)
{
  TSetDialog freq(this, 1);
  if (freq.Execute() == IDOK)
  {
    waveInStop(h);
    Init();
    SetupCaption();
    SetCaption(caption);
    waveInPrepareHeader(h, &buf, sizeof(buf));
    waveInAddBuffer(h, &buf, sizeof(buf));
    waveInStart(h);
  }
}

class TMainApp : public TApplication {
  public:
    TMainApp() : TApplication() {}
    void InitMainWindow();
};

void TMainApp::InitMainWindow()
{
  SetupCaption();
  TFrameWindow *W = new TFrameWindow(0, caption, new TMainWindow);
  W -> Attr.W = BUF_SIZE;
  W -> Attr.H = 160;
  W -> Attr.Style = WS_DLGFRAME | WS_SYSMENU | WS_MINIMIZEBOX;
  MainWindow = W;
}

int OwlMain(int, char* [])
{
  if (!waveInGetNumDevs)
  {
    MessageBox(0, "No waveform-audio input devices!", "Error", MB_OK + MB_ICONSTOP);
    return 1;
  }
  if (waveInGetDevCaps(0, &wavecaps, sizeof(wavecaps)) != MMSYSERR_NOERROR)
  {
    MessageBox(0, "Error retrieving waveform-audio caps!", "Error", MB_OK | MB_ICONSTOP);
    return 1;
  }
  return TMainApp().Run();
}

